from typing import *

import hug
import pandas as pd


def expose(func=None, output=hug.output_format.pretty_json):
    """Expose function over http and as a cli command."""

    def decorator(f):
        return hug.get(output=output)(hug.cli(output=output)(func))

    if func is not None:
        return decorator(func)
    else:
        return decorator


def to_records(df: pd.DataFrame) -> List[dict]:
    """Convert a datafram to a list of dictionaries minus the index."""
    return [
        {k: v for k, v in row.items() if k != "index"}
        for row in df.to_dict(orient="records")
    ]



"""
This API is a wrapper around Johns Hopkins' https://github.com/CSSEGISandData/COVID-19 dataset.

Please abide by their terms of use with respect to how you use their data via this API.

You can find the source code at https://github.com/knowsuchagency/covid-19
"""

import datetime as dt
import json
import logging
import threading
from typing import *

import hug

from covid_19.data import get_dataframe
from covid_19.utils import to_records, expose
from covid_19.wsgi import StandaloneApplication

df = get_dataframe()


def update_df(period=86400):
    """Update the dataframe daily."""
    global df
    event = threading.Event()
    while not event.wait(period):
        logging.warning("updating the dataframe")
        df = get_dataframe()


threading.Thread(target=update_df, daemon=True).start()

api = hug.API(__name__)

api.name = "COVID-19 API"


@hug.get(
    examples=[
        "country=US&date=2020-03-20",
        "min_date=2020-02-01&state=California&limit=100",
    ],
)
@hug.cli(output=hug.output_format.pretty_json)
def get_all(
    date=None,
    country=None,
    state=None,
    min_date=None,
    max_date=None,
    countries=None,
    states=None,
    limit=None,
):
    """
    Fetch all data from John Hopkins.

    Args:

        date (str): iso-formatted date
        country (str): country or region
        state (str): state or province
        min_date: (str): iso-formatted date
        max_date (str): iso-formatted date
        countries List[str]: json-formatted array of strings
        states List[str]: json-formatted array of strings
        limit (int): the maximum number of rows to be returned

    """
    result = df

    if date is not None:
        from pprint import pprint

        print(f"date: {date}")
        print(list(date))
        result = df[
            df["Last Update"].map(lambda d: d.date())
            == dt.datetime.fromisoformat(date).date()
        ]

    if country is not None:
        result = result[result["Country/Region"] == country]

    if state is not None:
        result = result[result["Province/State"] == state]

    if min_date is not None:
        result = result[
            result["Last Update"].map(lambda d: d.date())
            >= dt.datetime.fromisoformat(min_date).date()
        ]

    if max_date is not None:
        result = result[
            result["Last Update"].map(lambda d: d.date())
            <= dt.datetime.fromisoformat(max_date).date()
        ]

    if countries is not None:
        countries = (
            json.loads(countries)
            if not isinstance(countries, list)
            else countries
        )
        result = result[
            result["Countries/Regions"].map(lambda s: s in countries)
        ]

    if states is not None:
        states = json.loads(states) if not isinstance(states, list) else states
        result = result[result["Province/State"].map(lambda s: s in states)]

    records = to_records(result)

    if limit is not None:
        records = records[: int(limit)]

    return records


@expose
def countries():
    """Return all countries and regions in the dataset."""
    return df["Country/Region"].to_list()


@expose
def states():
    """Return all states and provinces in the dataset."""
    return to_records(
        df[["Province/State", "Country/Region"]].dropna(how="any")
    )


@expose
def for_date(date_string=None):
    """Return all data for a specific date."""
    date_string: Optional[str]

    date = (
        dt.datetime.fromisoformat(date_string).date()
        if date_string is not None
        else (dt.datetime.utcnow() - dt.timedelta(days=1)).date()
    )

    result = df[df["Last Update"].map(lambda d: d.date()) == date]

    return to_records(result)


@hug.cli()
def serve(host: str = "", port: int = 80):
    """Serve REST API locally."""
    options = {
        "bind": f"{host}:{port}",
        "worker-class": "egg:meinheld#gunicorn_worker",
    }
    StandaloneApplication(__hug_wsgi__, options).run()


if __name__ == "__main__":
    api.cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['covid_19']

package_data = \
{'': ['*']}

install_requires = \
['hug>=2.6.1,<3.0.0', 'pandas>=1.0.3,<2.0.0', 'requests>=2.23.0,<3.0.0']

entry_points = \
{'console_scripts': ['covid = covid_19.api:__hug__.cli']}

setup_kwargs = {
    'name': 'covid-19',
    'version': '0.3.4',
    'description': 'An API around the https://github.com/CSSEGISandData/COVID-19 dataset',
    'long_description': '# COVID-19 API\n\n**[coronavirus-api.net][coronavirus-api]**\n\n![](https://github.com/knowsuchagency/covid-19/workflows/black/badge.svg)\n![](https://github.com/knowsuchagency/covid-19/workflows/unit%20tests/badge.svg)\n\nThis API is a wrapper around Johns Hopkins\' https://github.com/CSSEGISandData/COVID-19 dataset.\n\nPlease abide by their terms of use with respect to how you use their data via this API.\n\n## Installation\n\nThis package is hosted on [pypi](https://pypi.org/project/covid-19/)\n\nThe recommended method of installation is through [pipx].\n```bash\npipx install covid-19\n```\nHowever, covid-19 can also be pip-installed as normal.\n```bash\npip install covid-19\n```\n\n## Website\n\nThe [coronavirus api website](https://coronavirus-api.net/get_all?min_date=2020-02-01&state=California&limit=100) allows several methods of slicing and dicing the data.\n\nFor example, the `get_all` endpoint (which returns all data) can be filtered for rows which only contain\ndata for California and Colorado as of March 22nd in this way:\n\n`https://coronavirus-api.net/get_all?min_date=2020-03-22&states=["California", "Colorada"]`\n\nThe [main page][coronavirus-api] will list the available endpoints and their available parameters.\n\n## CLI Usage\n\nThis package installs a command-line tool, `covid`\n\nIt lets you programmatically access John Hopkins\' dataset via terminal commands\nor via a rest api that can itself be instantiated locally from the cli\n\n```bash\ncovid --help\n\nThis API is a wrapper around Johns Hopkins\' https://github.com/CSSEGISandData/COVID-19 dataset.\n\nPlease abide by their terms of use with respect to how you use their data via this API.\n\n\nAvailable Commands:\n\n - get_allall: Fetch all data from John Hopkins.\n - countries: Return all countries and regions in the dataset.\n - states: Return all states and provinces in the dataset.\n - for_date: Return all data for a specific date.\n - serve: Serve REST API locally.\n\n```\n\ni.e.\n\n```bash\ncovid for_date 2020-03-21\n[\n    {\n        "Province/State": "Hubei",\n        "Country/Region": "China",\n        "Last Update": "2020-03-22T09:43:06",\n        "Confirmed": 67800.0,\n        "Deaths": 3144.0,\n        "Recovered": 59433.0,\n        "Latitude": 30.9756,\n        "Longitude": 112.2707\n...\n```\n\n## Docker\n\nThis package can also be run as a [docker image][docker image].\n\n```bash\ndocker run knowsuchagency/covid-19 --help\n```\n\n[pipx]: https://github.com/pipxproject/pipx\n[coronavirus-api]: https://coronavirus-api.net\n[docker image]: https://hub.docker.com/repository/docker/knowsuchagency/covid-19\n',
    'author': 'Stephan Fitzpatrick',
    'author_email': 'knowsuchagency@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://coronavirus-api.net',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import datetime as dt
import io
from concurrent import futures

import pandas as pd
import requests

INITIAL_DATE = dt.date(2020, 1, 22)


def get_dataframe_for_date(date: dt.date = INITIAL_DATE) -> pd.DataFrame:
    """Fetch the appropriate csv from github and return it as a datafram."""

    url = (
        "https://raw.githubusercontent.com/"
        "CSSEGISandData/COVID-19/master/"
        "csse_covid_19_data/"
        "csse_covid_19_daily_reports/"
        "{date}.csv"
    ).format(date=date.strftime("%m-%d-%Y"))

    resp = requests.get(url)

    with io.StringIO(resp.text) as fp:
        return pd.read_csv(fp)


def get_dataframe(start_date=INITIAL_DATE, end_date=None):
    """Fetch the appropriate csv(s) for a given daterange, concatenate them, and return a csv."""

    end_date = (
        end_date
        if end_date is not None
        else dt.datetime.utcnow() - dt.timedelta(days=1)
    ).date()

    dates = pd.date_range(start_date, end_date)

    with futures.ThreadPoolExecutor() as ex:

        df = pd.concat(ex.map(get_dataframe_for_date, dates))

        df["Last Update"] = df["Last Update"].astype("datetime64[ns]")

    return df
